package Beastify;

use strict;

# {{{ Functions to clean out (#ifdef|#if)/#endif sections, very dumb application 
sub cleanout_ifdefs
{
	my ($src, $symbol, $skipelse) = @_;

	my @lines = split /\n/, $src;
	my @result;
	my $skipping = 0;
	my $level = 0; 
	foreach my $l (@lines) {
		if ($level) {
			if ($l =~ /^\s*#\s*endif/) {
				$level--;
				if ($level == 0) {
					my $pass_this = $skipping;
					$skipping = 0;
					next if($pass_this);
				}
			}
			$level++ if ($l =~ /^\s*#\s*if/);
			if ($l =~ /^\s*#\s*else/) {
				if ($level == 1) {
					unless ($skipelse) {
						push @result, "#if 1 /* Put by beastify scipt for symbol $symbol */"; 
						$skipping = 0;
						next;
					}
				}
			}

			$skipping = 0 if($level == 0);
		} 
		else {
			if ($l =~ /\s*#\s*ifdef\s*$symbol\s*$/) {
				$level = 1;
				$skipping = 1;
			}
			elsif ($l =~ /\s*#\s*if\s+defined\s*\(\s*$symbol\s*\)\s*$/) {
				$level = 1;
				$skipping = 1;
			}
		}

		push @result, $l unless($skipping);
	}

	return join ("\n", @result) . "\n";
}

sub cleanout_if_with_str
{
	my ($src, $str, $skipelse) = @_;

	my @lines = split /\n/, $src;
	my @result;
	my $skipping = 0;
	my $level = 0; 
	foreach my $l (@lines) {
		if ($level) {
			if ($l =~ /^\s*#\s*endif/) {
				$level--;
				if ($level == 0) {
					my $pass_this = $skipping;
					$skipping = 0;
					next if($pass_this);
				}
			}
			$level++ if ($l =~ /^\s*#\s*if/);
			if ($l =~ /^\s*#\s*else/) {
				if ($level == 1) {
					unless ($skipelse) {
						push @result, "#if 1 /* Put by beastify scipt for substr '$str' */"; 
						$skipping = 0;
						next;
					}
				}
			}

			$skipping = 0 if($level == 0);
		} 
		else {
			if ($l =~ /\s*#\s*if\s+.*$str.*$/) {
				$level = 1;
				$skipping = 1;
			}
		}

		push @result, $l unless($skipping);
	}

	return join ("\n", @result) . "\n";
}

# }}}
sub port_locks # {{{ 
{
	my $src = shift;

	$src =~ s/spinlock_t/lock_t/g;
	$src =~ s/ = [A-Z]+_LOCK_UNLOCKED//g;
	$src =~ s/spin_lock_irqsave\(&(.*), [a-z_0-9]+\)/ZAP_LOCK($1)/g;
	$src =~ s/spin_lock\(&(.*)\)/ZAP_LOCK($1)/g;
	$src =~ s/spin_lock_init\s*\(\s*\&(.*)\s*\)/ZAP_LOCK_INIT($1, "$1")/g;
	$src =~ s/spin_unlock_irqrestore\(&(.*), [a-z_0-9]+\)/ZAP_UNLOCK($1)/g;
	$src =~ s/spin_unlock\(&(.*)\)/ZAP_UNLOCK($1)/g;
	$src =~ s/write_lock_irqsave\(&(.*), ?flagso?\)/ZAP_RWLOCK_LOCK_EXCLUSIVE($1)/g;
	$src =~ s/write_unlock_irqrestore\(&(.*), ?flagso?\)/ZAP_RWLOCK_UNLOCK_EXCLUSIVE($1)/g;
	$src =~ s/write_lock\(&(.*)\)/ZAP_RWLOCK_LOCK_EXCLUSIVE($1)/g;
	$src =~ s/write_unlock\(&(.*)\)/ZAP_RWLOCK_UNLOCK_EXCLUSIVE($1)/g;

	$src =~ s/read_lock\(&(.*)\)/ZAP_RWLOCK_LOCK_SHARED($1)/g;
	$src =~ s/read_trylock\(&(.*)\)/ZAP_RWLOCK_TRYLOCK_SHARED($1)/g;
	$src =~ s/read_unlock\(&(.*)\)/ZAP_RWLOCK_UNLOCK_SHARED($1)/g;

	#remove flags/flagso variables 
	# $src =~ s/\s+unsigned long flags;./\n/gs;
	$src =~ s/\s+unsigned long flags, flagso;./\n/gs;

	return $src;
}
# }}}
sub port_printk # {{{
{
	my $src = shift;
	# printk is penguinism
	$src =~ s/printk/printf/g;
	# clean out log level left from printk
	$src =~ s/\(\s*KERN_[A-Z]+\s*/(/g;
	return $src;
}
# }}}
sub port_errno # {{{
{
	my $src = shift;
	# BSD uses "return EXXX" not "return -EXXX"
	# 2 is for E2BIG 
	$src =~ s/-(\s*E[A-Z2]+\)?)/$1/g;
	return $src;
}
# }}}
sub port_interrupt_code # {{{
{
	my $src = shift;
	my $return_code = <<__EOC__;
#if __FreeBSD_version < 700031
		return;
#else
		return FILTER_HANDLED;
#endif
__EOC__

	$src =~ s/(#ifdef\s+LINUX26\s+)?[^\n]*return IRQ_[^\n]+;(\s*#else\s+return;)?(\s+#endif)?/$return_code/g;
	return $src;
}
# }}}
sub port_types # {{{
{
	my $src = shift;
	$src =~ s/__u(8|16|32)/uint$1_t/g;
	$src =~ s/__s(8|16|32)/int$1_t/g;
	return $src;
}
# }}}
sub port_kmemory # {{{
{
	my ($src, $class) = @_;
	$src =~ s/kmalloc\((.*), ?GFP_[A-Z]+\)/malloc($1, $class, M_NOWAIT)/g;
	$src =~ s/kfree\((.*)\);/free($1, $class);/g;
	return $src;
}
# }}}
sub cleanout_moduleinfo # {{{
{
	my $src = shift;
	$src = cleanout_ifdefs($src, "MODULE_LICENSE"); 
	$src = cleanout_ifdefs($src, "MODULE_VERSION"); 
	$src = cleanout_ifdefs($src, "MODULE_ALIAS"); 
	$src =~ s/MODULE_AUTHOR(.*);\n//;
	$src =~ s/MODULE_DESCRIPTION(.*);\n//;
	$src =~ s/module_init.*\n//;
	$src =~ s/module_exit.*\n//;
	return $src;
}
# }}}
sub port_tunables # {{{
{
	my ($src, $prefix) = @_;
	$src =~ s/#ifdef LINUX26\n(?:module_param\([^\n]*\);\s*|#ifdef[^\n]*\n|#endif\n)+#else\s*(.*?)#endif/$1/s;
	$src =~ s/MODULE_PARM\((.*), "i"\)/TUNABLE_INT("$prefix.$1", \&$1)/g;
	$src =~ s/MODULE_PARM\((.*), "s"\)/TUNABLE_STR("$prefix.$1", $1, sizeof($1))/g;
	return $src;
}
# }}}
sub generate_module_wrapper  # {{{
{
	my ($src, $module, $method_prefix, $softc_struct) = @_;
	$src =~ s/__exit //s;
	$src =~ s/__init //s;
	$src =~ s/zap_pci_module\([&a-z0-9_]+\)/0 \/* by beastify *\//g;
	$src =~ s/.*pci_unregister_driver.*\n//;
	# Cleanout pci_driver func
	$src =~ s/(static )?struct pci_driver.*;\n//g;
	$src =~ s/(static )?struct pci_driver.*?\n};//s;
	$src =~ s/MODULE_DEVICE_TABLE\(pci, ?([a-z_][a-z_0-9]+)\);//;
	my $pci_ids = $1;
	warn "No PCI ids table" unless(defined($pci_ids));

	$src .= "HWDRIVER_MODULE_HANDLER($method_prefix);\n";
	$src .= "HWDRIVER_METHODS($method_prefix, $1, ${method_prefix}_init_one, ${method_prefix}_remove_one);\n";
	$src .= "DEFINE_HWDRIVER($module, $method_prefix, $softc_struct, ${method_prefix}_load);\n";

	return $src;
}
# }}}
sub port_pci_stuff # {{{
{
	my $src = shift;
	$src =~ s/struct\s+pci_dev\s*\*/device_t /g;
	$src =~ s/(wc->dev|p?dev)->bus->number/pci_get_bus($1)/g;
	$src =~ s/PCI_SLOT\((wc->dev|p?dev)->devfn\) \+ 1/pci_get_slot($1)/g;
	$src =~ s/(wc->dev|p?dev)->irq/pci_get_irq($1)/g;
	$src =~ s/pci_read_config_byte\((.*), (.*), \&(.*?)\)/$3 = pci_read_config($1, $2, 1)/g;
	$src =~ s/pci_read_config_dword\((.*), (.*), \&(.*?)\)/$3 = pci_read_config($1, $2, 4)/g;
	$src =~ s/pci_write_config_dword\((.*), (.*), (.*)\)/pci_write_config($1, $2, $3, 4)/g;
	$src =~ s/PCI_COMMAND_IO/PCIM_CMD_PORTEN/g;
	$src =~ s/PCI_COMMAND_MEMORY/PCIM_CMD_MEMEN/g;
	$src =~ s/PCI_COMMAND_MASTER/PCIM_CMD_BUSMASTEREN/g;
	$src =~ s/PCI_COMMAND/PCIR_COMMAND/g;
	return $src;
}
# }}}
sub port_jiffies # {{{
{
	my $src = shift;
	$src =~ s/ *= *jiffies;/ = jiffies();/g;
	$src =~ s/( |\()jiffies,/$1jiffies(),/g;
	$src =~ s/(\(|\s|,)jiffies\s*(\+|-|<|\))\s*/$1jiffies() $2/g;
	$src =~ s/HZ/hz/g;
	return $src;
}
# }}}
sub port_iospace # {{{
{
	my $src = shift;
	$src =~ s/outb\((.*), \w+->io(addr|base) \+ (.*)\);/WRITE1($3, $1);/g;
	$src =~ s/outl\((.*),\s+\w+->io(addr|base) \+ (.*)\);/WRITE4($3, $1);/g;
	$src =~ s/inb\(\w+->io(addr|base) \+ /READ1(/g;
	$src =~ s/inl\(\w+->io(addr|base) \+ /READ4(/g;
	return $src;
}
# }}}
sub port_module_generics # {{{
{
	my $src = shift;
	$src =~ s/wc = k?malloc\(.*?,[^\)]*\)/wc = device_get_softc(pdev)/; # in condition
	$src =~ s/.*k?free\(wc.*\n//g; # It's kfree yet
	$src =~ s/wc->(read|write|descrip)chunk/wc->wc_dma.$1chunk/g;
	$src =~ s/wc->(read|write|descrip)dma/wc->wc_dma.$1_dmaaddr/g;
	$src = port_iospace($src);

	$src = port_jiffies($src);
	$src = port_pci_stuff($src);
	$src =~ s/(?:\s+init_waitqueue_head(.*?);.)+/\n/gs;
	$src =~ s/strncat/strlcat/g;
	$src =~ s/wmb\(\);/\/* TODO: wmb(); *\//g;
	# Workqueues
	$src =~ s/struct work_struct/struct task/g; #XXX
	$src =~ s/struct workqueue_struct/struct taskqueue/g; #XXX
	# pre-current requires proc
	# $src =~ s/(\s+)(.*?)\s*=\s*create_singlethread_workqueue\((.*)\)/$1$2 = taskqueue_create($3, M_NOWAIT | M_ZERO, taskqueue_thread_enqueue, wc, &$2_proc);$1taskqueue_start_threads(&$2_proc, 1, PI_NET, "zaptel taskq");/;
	$src =~ s/(\s+)(.*)->(.*?)\s*=\s*create_singlethread_workqueue\((.*)\)/$1$2->$3 = taskqueue_create($4, M_NOWAIT | M_ZERO, taskqueue_thread_enqueue, $2);$1taskqueue_start_threads(&$2->$3, 1, PI_NET, "zaptel taskq");/;
	$src =~ s/INIT_WORK\((.*), (.*), (.*)\)/TASK_INIT($1, PI_NET, $2, $3)/g;
	$src =~ s/queue_work/taskqueue_enqueue/g;
	$src =~ s/flush_workqueue\((.*)->(.*)\)/#error FIXME: taskqueue_drain($1->$2, &$1->work)/g;
	$src =~ s/destroy_workqueue/taskqueue_free/g;

	$src =~ s/ __dev(init|exit) / /g;
	$src =~ s/if\s*\(pci_enable_device\(pdev\)\)(\s*{?)/if (pci_enable_io(pdev, SYS_RES_IOPORT))$1/;
	$src =~ s/wc->io(addr|base) = pci_resource_start.*\n//;
	$src =~ s/request_region\([^\)]+\)/zt_map_iospace(pdev, \&wc->wc_io)/;
	$src =~ s/(\s+).*pci_alloc_consistent\(pdev, (.*), .*\);/$1zt_dma_allocate(pdev, &wc->wc_dma, $2);/;
	$src =~ s/release_region.*/zt_unmap_iospace(wc->dev, &wc->wc_io);/g;
	$src =~ s/free_irq.*/zt_teardown_irq(wc->dev, &wc->wc_irq);/g;
	$src =~ s/pci_free_consistent.*/zt_dma_destroy(&wc->wc_dma);/g;
	$src =~ s/pci_set_master/pci_enable_busmaster/g;
	$src =~ s/.*\n.*pci_set_drvdata.*\n//g;
	$src =~ s/pci_get_drvdata/device_get_softc/g;
	$src =~ s/request_irq\([^,]+, ([^,]+),[^\)]+\)/zt_setup_irq(pdev, &wc->wc_irq, $1, wc)/;
	# Remove reference to pdev->irq
	$src =~ s/.*Unable to request IRQ.*\n//;

	# pci ids table
	$src =~ s/0, 0, \(unsigned long\)/0, 0,/g;


	return $src;
}
# }}}
sub port_ioctl # {{{
{
	my $src = shift;
	# ioctl stuff
	$src =~ s/if \(get_user\((.*), ?(.*)\)\)/if (copyin($2, \&$1, sizeof($1)))/g;
	$src =~ s/if \(copy_to_user\((.*), ?(.*), ?(.*)\)\)\s*\n?\s+return\s+E[A-Z2]+;/bcopy($2, $1, $3);/g;
	$src =~ s/if \(copy_from_user\((.*), ?(.*), ?(.*)\)\)[ \t\\]*\n?\s+return\s+E[A-Z2]+;/bcopy($2, $1, $3);/g;
	# ioctl prototype
	$src =~ s/unsigned long data/caddr_t data/g;
	return $src;

}
# }}}
1;
