package wctdm24xxp;
use strict;
use Beastify;

our (%files_to_copy, %files_to_beastify);

my $includes = <<__EOF__;
#include <sys/param.h>  /* defines used in kernel.h */
#include <sys/module.h>
#include <sys/systm.h>
#include <sys/errno.h>
#include <sys/kernel.h> /* types used in module initialization */
#include <sys/conf.h>   /* cdevsw struct */
#include <sys/uio.h>    /* uio struct */
#include <sys/malloc.h>
#include <sys/lock.h>
#include <sys/sema.h>
#include <sys/endian.h>
#include <sys/bus.h>  /* structs, prototypes for pci bus stuff */

#include <machine/bus.h>
#include <sys/rman.h>
#include <machine/resource.h>
#include <machine/clock.h>
#include <sys/sysctl.h>
#include <sys/linker.h>
#include <sys/firmware.h>

#include <dev/pci/pcireg.h>
#include <dev/pci/pcivar.h>

MALLOC_DEFINE(M_WCTDM24, "wctdm24", "wctdm24 hw data structures");

__EOF__

my $gpak_includes = <<__EOF__;
#include <sys/param.h>  /* defines used in kernel.h */
#include <sys/module.h>
#include <sys/systm.h>
#include <sys/lock.h>
#include <sys/mutex.h>
#include <sys/sema.h>
#include <sys/bus.h>  /* structs, prototypes for pci bus stuff */
#include <sys/linker.h>
#include <sys/firmware.h>

#include <machine/bus.h>
__EOF__



my $externs = <<__EOC__;
#if !defined(HOTPLUG_FIRMWARE)
/* dummies, firmware stuff is in TODO */ 
int _binary_zaptel_fw_vpmadt032_bin_size;
void * _binary_zaptel_fw_vpmadt032_bin_start;
#else 
static const char vpmadt032_firmware[] = "zaptel-fw-vpmadt032.bin";
#endif
__EOC__

sub fixup_base
{
	my $src = shift;

	$src =~ s/static char \*opermode/static char opermode[128]/;
	$src = Beastify::port_locks($src);
	$src = Beastify::port_printk($src);
	$src = Beastify::port_errno($src);
	$src =~ s@#include\s+<linux/.*#include "zaptel.h"@$includes#include "zaptel.h"@s;
	$src =~ s/int schluffen.*?\n}//s;
	$src =~ s/wake_up_interruptible/wakeup/g;
	$src =~ s/([ \t]+)#define MAX (\d+)/$1#undef MAX\n$1#define MAX $2/;
	$src =~ s/dma_addr_t/bus_addr_t/g;
	$src =~ s/void vpm150m_set_chanconfig_from_state/static void vpm150m_set_chanconfig_from_state/g;
	$src =~ s/void handle_/static void handle_/g;
	$src =~ s/#if LINUX_VERSION_CODE < KERNEL_VERSION[^\n]*\n(.*?)#else.*?#endif/$1/gs;
	$src =~ s/static void vpm150m_bh\(void \*data\)/static void vpm150m_bh(void \*data, int pending)/;

	# remove extern devinitions from vpmadt032_init_result
	$src =~ s/#if !defined\(HOTPLUG_FIRMWARE\).*?#else\n\tstatic.*?#endif//s;
	# and add them to the beginning
	$src =~ s/(static int loopcurrent.*\n)/$1\n$externs/;
	# More firmware stuff coming
	$src =~ s/firmware\.size/firmware\.datasize/g;
	$src =~ s/release_firmware\((.*)\)/firmware_unregister($1->name)/g;
	# semaphores
	$src =~ s/sema_init\((.*)\)/sema_init($1, "wctdm semaphore")/g;
	$src =~ s/vpmsupport = 1/vpmsupport = 0/;
	$src =~ s/wctdm_driver.name/"wctdm24xxp"/g;

	$src = Beastify::port_interrupt_code($src);
	$src = Beastify::port_types($src);
	$src = Beastify::port_ioctl($src);
	$src = Beastify::port_kmemory($src, "M_WCTDM24");

	$src = Beastify::port_module_generics($src);
	$src = Beastify::cleanout_moduleinfo($src);
	$src = Beastify::port_tunables($src, "zaptel.wctdm24xxp");
	$src = Beastify::generate_module_wrapper($src, "wctdm24xxp", "wctdm", "wctdm");

	#  Add ifaceslock initialization
	$src =~ s/(static int wctdm_init.*int x;\n)/$1\n\tZAP_LOCK_INIT(ifacelock, "ifacelock");\n/s;
	$src =~ s/(static void wctdm_cleanup.*?{\n)/$1\n\tZAP_LOCK_DESTROY(ifacelock);\n/s;

	# unused variable
	$src =~ s/(static enum vpmadt032_init_result wctdm_vpm150m_init\(.*?)device_t  pdev[^\n]*;/$1/s;

	return $src;
}

sub fixup_h
{
	my $src = shift;
	$src = Beastify::cleanout_if_with_str($src, "LINUX_VERSION_CODE", 1);
	$src = Beastify::port_locks($src);
	$src =~ s/"\.\.\//"/g;
	$src =~ s/#include <asm.*\n//g;
	$src =~ s/struct semaphore/struct sema/g;
	$src =~ s/#define VPM_SUPPORT/#define VPM_SUPPORT\n#define VPM150M_SUPPORT/g;
	$src =~ s/wait_queue_head_t(\s+)/void$1*/g;
	$src =~ s/struct work_struct/struct task/g; #XXX
	$src =~ s/struct workqueue_struct/struct taskqueue/g; #XXX
	$src =~ s/(\s+)(struct taskqueue)/$1struct proc *wq_proc;\n$1$2/g;
	$src =~ s/int schluffen.*\n//;
	$src =~ s@([\t ]+)unsigned long iobase;.*?volatile unsigned int \*descripchunk;[^\n]*@$1struct zt_io_t wc_io;\n$1struct zt_dma_t wc_dma;\n$1struct zt_irq_t wc_irq;@s;
	$src =~ s/struct\s+pci_dev\s*\*/device_t /g;
	return $src;
}

sub fixup_gpakcust_h
{
	my $src = shift;
	$src = Beastify::cleanout_ifdefs($src, "VPM150M_SUPPORT");
	return $src;
}

sub fixup_gpakcust_c
{
	my $src = shift;
	$src =~ s@#include\s+<linux/.*#include "zaptel.h"@$gpak_includes#include "zaptel.h"@s;
	$src = Beastify::port_locks($src);
	$src =~ s/int wctdm_vpm150m_(get|set)reg_full/static int wctdm_vpm150m_$1reg_full/g;
	$src = Beastify::port_printk($src);
	$src =~ s/down_interruptible/sema_wait/g;
	$src =~ s/up\(/sema_post\(/g;
	$src =~ s/fw->size/fw->datasize/g;
	$src =~ s/fw->fw->data\[/((const uint8_t *)fw->fw->data)\[/g;

	return $src;
}

sub fixup_gpakapi_c
{
	my $src = shift;
	$src =~ s@#include\s+<linux/.*#include "zaptel.h"@$gpak_includes#include "zaptel.h"@s;

	return $src;
}

# {{{ Files to port for wctdm24xxp package
%files_to_copy = (
	"kernel/wctdm24xxp/gpakenum.h" => "wctdm24xxp/gpakenum.h",
	"kernel/wctdm24xxp/GpakApi.h" => "wctdm24xxp/GpakApi.h",
	"kernel/wctdm24xxp/GpakHpi.h" => "wctdm24xxp/GpakHpi.h",
	"kernel/wctdm24xxp/gpakErrs.h" => "wctdm24xxp/gpakErrs.h",
);

%files_to_beastify = (
	"kernel/wctdm24xxp/base.c" => {
		filter => \&fixup_base,
		dst => "wctdm24xxp/base.c",
	},
	"kernel/wctdm24xxp/wctdm24xxp.h" => {
		filter => \&fixup_h,
		dst => "wctdm24xxp/wctdm24xxp.h",
	},
	"kernel/wctdm24xxp/GpakCust.h" => {
		filter => \&fixup_gpakcust_h,
		dst => "wctdm24xxp/GpakCust.h",
	},
	"kernel/wctdm24xxp/GpakCust.c" => {
		filter => \&fixup_gpakcust_c,
		dst => "wctdm24xxp/GpakCust.c",
	},
	"kernel/wctdm24xxp/GpakApi.c" => {
		filter => \&fixup_gpakapi_c,
		dst => "wctdm24xxp/GpakApi.c",
	},
); 
# }}}

1;
