package wcte11xp;
use strict;
use Beastify;

our (%files_to_copy, %files_to_beastify);

my $includes = <<__EOF__;
#include <sys/param.h>  /* defines used in kernel.h */
#include <sys/module.h>
#include <sys/systm.h>
#include <sys/errno.h>
#include <sys/kernel.h> /* types used in module initialization */
#include <sys/conf.h>   /* cdevsw struct */
#include <sys/uio.h>    /* uio struct */
#include <sys/malloc.h>
#include <sys/lock.h>
#include <sys/sema.h>
#include <sys/endian.h>
#include <sys/bus.h>  /* structs, prototypes for pci bus stuff */
#include <sys/ioccom.h>

#include <machine/bus.h>
#include <sys/rman.h>
#include <machine/resource.h>
#include <machine/clock.h>
#include <sys/sysctl.h>
#include <sys/linker.h>
#include <sys/firmware.h>

#include <dev/pci/pcireg.h>
#include <dev/pci/pcivar.h>

MALLOC_DEFINE(M_WCTE11XP, "wcte11xp", "wcte11xp hw data structures");

__EOF__

sub fixup_wcte11xp
{
	my $src = shift;

	# definitions stuff
	$src =~ s@([\t ]+)dma_addr_t\s+readdma;.*?volatile unsigned char \*readchunk;[^\n]*@$1struct zt_dma_t wc_dma;\n$1struct zt_irq_t wc_irq;@s;
	$src =~ s@([\t ]+)unsigned long ioaddr;@$1struct zt_io_t wc_io;@s;
	$src =~ s/struct\s+pci_dev\s*\*/device_t /g;

	# Other stuff
	$src =~ s/static char \*opermode/static char opermode[128]/;
	$src = Beastify::port_locks($src);
	$src = Beastify::port_printk($src);
	$src = Beastify::port_errno($src);
	$src =~ s@#include\s+<linux/.*#include "zaptel.h"@$includes#include "zaptel.h"@s;
	$src =~ s/int schluffen.*?\n}//s;
	$src =~ s/wake_up_interruptible/wakeup/g;
	$src =~ s/([ \t]+)#define MAX (\d+)/$1#undef MAX\n$1#define MAX $2/;
	$src =~ s/dma_addr_t/bus_addr_t/g;
	$src =~ s/#if LINUX_VERSION_CODE < KERNEL_VERSION[^\n]*\n(.*?)#else.*?#endif/$1/gs;

	$src = Beastify::port_interrupt_code($src);
	$src = Beastify::port_types($src);
	$src = Beastify::port_ioctl($src);
	$src = Beastify::port_kmemory($src, "M_WCTE11XP");

	$src = Beastify::port_module_generics($src);
	$src = Beastify::cleanout_moduleinfo($src);
	$src = Beastify::port_tunables($src, "zaptel.wcte11xp");
	$src = Beastify::generate_module_wrapper($src, "wcte11xp", "te11xp", "t1");

	# replace schedule_timeout with tsleep
	$src =~ s/\s+set_current_state.*//;
	$src =~ s/schedule_timeout.*/tsleep(wc, PZERO | PCATCH, "waitintr", 2);/;

	#  Add ifaceslock initialization
	$src =~ s/(\s+)(return res \? ENODEV : 0;)/$1ZAP_LOCK_INIT(ifacelock, "ifacelock");\n$1$2/;
	$src =~ s/(static void te11xp_cleanup.*?{\n)/$1\n\tZAP_LOCK_DESTROY(ifacelock);\n/s;
	$src =~ s/module_printf\("/debug_printf(1, "/g;

	return $src;
}

# {{{ Files to port for wcte11xp package
%files_to_copy = (
);

%files_to_beastify = (
	"kernel/wcte11xp.c" => {
		filter => \&fixup_wcte11xp,
		dst => "wcte11xp/wcte11xp.c",
	},
); 
# }}}

1;
