package zaptel;
use Beastify;
use zaptel_bits;

our (%files_to_copy, %files_to_beastify);

# {{{ fixups for prototypes and function arguments
my %prototypes_fixup = (
	"static int zt_open" => "(struct cdev *dev, int flag, int mode, struct thread * td)",
	"static int zt_timing_open" => "(struct cdev *dev)",
	"static int zt_ctl_open" => "(struct cdev *dev)",
	"static int zt_chan_open" => "(struct cdev *dev)",
	"static int zt_specchan_open" => "(struct cdev *dev, int unit, int inc)",
	"static ssize_t zt_chan_read" => "(struct uio * uio, int unit, int fflag)",
	"static ssize_t zt_chan_write" => "(struct uio * uio, int unit, int fflag)",
	"static int zt_chan_close" => "(struct cdev *dev)",
	"static int zt_timer_close" => "(struct cdev *dev)",
	"static int zt_specchan_close" => "(struct cdev *dev, int unit)",
	"static int zt_ctl_close" => "(struct cdev *dev)",
	"static int zt_close" => "(struct cdev *dev, int flag, int otyp, struct thread *td)",
	"int zt_read" => "(struct cdev *dev, struct uio *uio, int fflag)",
	"int zt_write" => "(struct cdev *dev, struct uio *uio, int fflag)",
	"int zt_poll" => "(struct cdev *dev, int events, struct thread *td)",
	"static unsigned int zt_timer_poll" => "(struct cdev *dev, int events, struct thread *td)",
	"static unsigned int\nzt_chan_poll" => "(struct cdev *dev, int events, struct thread *td, int unit)",
	"static int zt_ioctl" => "(struct cdev *dev, u_long cmd, caddr_t arg, int fflag, struct thread *td)",
	"static int zt_ctl_ioctl" => "(struct cdev *dev, u_long cmd, caddr_t arg, int fflag)",
	"static int zt_timer_ioctl" => "(struct cdev *dev, u_long cmd, caddr_t arg, int fflag, struct zt_timer * timer)",
	"static int zt_common_ioctl" => "(struct cdev *dev, u_long cmd, caddr_t arg, int fflag, int unit)",
	"static int zt_chanandpseudo_ioctl" => "(struct cdev *dev, u_long cmd, caddr_t arg, int fflag, int unit)",
	"static int zt_chan_ioctl" => "(struct cdev *dev, u_long cmd, caddr_t arg, int fflag, int unit)",
	"static int zt_prechan_ioctl" => "(struct cdev *dev, u_long cmd, caddr_t arg, int fflag, int unit)",
	"static int ioctl_echocancel" => "(struct zt_chan *chan, struct zt_echocanparams *ecp, caddr_t arg)",
	"int zt_mmap" => "(struct cdev *dev, vm_offset_t offset, vm_paddr_t *paddr, int nprot)",
	);

my %args_fixup = (
	'\(inode, file\)' => "(dev)",
	'\(inode, file, chan->channo, 1\)' => "(dev, chan->channo, 1)",
	'\(inode, file, unit, 1\)' => "(dev, unit, 1)",
	'\(file,\s*wait_table' => "(dev, events, td",
	'\(file, usrbuf, count, chan->channo\)' => "(uio, chan->channo, fflag)",
	'\(file, usrbuf, count, unit\)' => "(uio, unit, fflag)",
	'\(inode, file, cmd, data' => "(dev, cmd, arg, fflag",
	'\(inode, file, cmd, data' => "(dev, cmd, arg, fflag",
	'\(cmd, data' => "(cmd, arg",
	'\(chan, data' => "(chan, arg",
	'\(chan, cmd, data\)' => "(chan, cmd, arg)",
	'\(inode, file' => "(dev", # all other cases

);
# }}}

# Filters 
sub zaptel_base_filter # {{{ 
{
	my $src = shift;
	# XXX: Just use *BSD names convention
	$src =~ s/zt_([a-z_]*)release/zt_$1close/g;

	# fix locks
	$src = Beastify::port_locks($src);
	$src = Beastify::port_errno($src);

	# Use EINVAL instead of ENODATA since FreeBSD has no such entry in errno.h
	$src =~ s/ENODATA/EINVAL/g;

#types
	$src =~ s/__u(8|16|32)/uint$1_t/g;


	$src = Beastify::port_printk($src);
	$src =~ s/\(\s*KERN_[A-Z]+\s*/(/g;

	# Use malloc(9) interface
	$src = Beastify::port_kmemory($src, "M_ZAP");
	$src =~ s/memmove\((.*), ?(.*), ?(.*)\)/ovbcopy($2, $1, $3)/g;

# wakeup
	$src =~ s/wake_up_interruptible/wakeup/g;
	$src =~ s/(?:\s+init_waitqueue_head(.*?);.)+/\n/gs;
# zaptel specific: we're waking up for poll from fast taskqueue
	$src =~ s/wait_queue_head_t sel;/struct selinfo\tsel;\n\tstruct task\tselwakeup_task;/;
	$src =~ s/wakeup\(\&(.*)->sel\);/taskqueue_enqueue_fast(taskqueue_fast, &$1->selwakeup_task);/g;


# ioctl stuff
	$src =~ s/if \(copy_from_user\((.*), ?(.*), ?(.*)\)\)[ \t\\]*\n\s+return\s+E[A-Z2]+;/bcopy($2, $1, $3);/g;
	$src =~ s/if \(copy_from_user\((.*), ?(.*), ?(.*)\)\)[ \t\\]*{/if (copyin($2, $1, $3)) {/g;
	$src =~ s/if \(copy_to_user\((.*), ?(.*), ?(.*)\)\)\s*\n\s+return\s+E[A-Z2]+;/bcopy($2, $1, $3);/g;
# $src =~ s/if \(copy_to_user\((.*), ?(.*), ?(.*)\)\)/if (copyout($2, $1, $3))/g;
	$src =~ s/put_user\((.*), ?(.*)\)/bcopy(\&$1, $2, sizeof($1))/g;
	$src =~ s/if \(get_user\((.*), ?(.*)\)\)[ \t\\]*\n\s+return\s+E[A-Z2]+;/bcopy($2, \&$1, sizeof($1));/g; # two exception, I'm to lazy to do it properly
	$src =~ s/get_user\((.*), ?(.*)\)/bcopy($2, \&$1, sizeof($1))/g;
# in ioctl routine we use caddr_t arg (specific only for zaptel)
	$src =~ s/unsigned long data/caddr_t arg/g;
	$src =~ s/\(unsigned long\) ind\.data/(caddr_t)ind.data, fflag/g;
# $src =~ s/\(data\)/\(arg\)/g;
	$src =~ s/(, *|\( *)\([a-z _]+ ?\*\) ?data,/$1arg,/g;
	$src =~ s/ioctl_load_zone\(data/ioctl_load_zone(arg/;
	$src =~ s/#ifndef\s+LINUX26\n(\s+MOD_(?:INC|DEC)_USE_COUNT;)\s+#endif/$1/g;
	# Fix out other data mentions in ioctl code
	$src =~ s/(case ZT_SETCADENCE:\n.*)data(.*)/$1arg$2/;
	$src =~ s/\(void \*\) data/arg/g;


	# Comment out transcode ops
	$src =~ s/(request_module.*)/\/* XXX:XXX:  $1 *\//g;
	$src =~ s/(.*unit\s*==\s*250[^\n{]*)\n(\s*).*/$1\n$2return ENXIO;/g;
	$src =~ s/(\s+)(if\s*\(unit\s*==\s*250\) {)\s+(.*?)\1}/$1$2$1\treturn ENXIO;\n#if 0$1\t$3\n#endif$1}/gs;
	$src =~ s/\s+int ret = ENXIO;\n/\n/;
	$src =~ s/\s+if \(count < 0\).*?return EINVAL;//gs;

	$src =~ s/z = ptr = slab = malloc/ptr = slab = (char *)malloc/;
	$src =~ s/([ \t]+)(memset\(slab)/$1z = (void *) slab;\n$1$2/;

# fix all Linux'es CONFIG_
# $src = Beastify::cleanout_ifdefs($src, "CONFIG_PROC_FS"); 
	$src = Beastify::cleanout_ifdefs($src, "CONFIG_DEVFS_FS", 1); 
	$src = Beastify::cleanout_ifdefs($src, "CONFIG_ZAP_UDEV"); 
	$src = Beastify::cleanout_ifdefs($src, "DEFINE_SPINLOCK"); 
	$src = Beastify::cleanout_ifdefs($src, "DEFINE_RWLOCK"); 
# $src = Beastify::cleanout_ifdefs($src, "CONFIG_CACL_XLAW"); 
	$src = Beastify::cleanout_moduleinfo($src);
# cleanout __init and __exit labels
	$src =~ s/\s+__(init|exit)//g;

# we use si_drv1 instead of file->private_data
	$src =~ s/file->private_data/dev->si_drv1/g;

# fixup includes 
	$src =~ s/#include "zconfig.h".*#define __ECHO_STATE_MUTE/$zaptel_bits::includes#define __ECHO_STATE_MUTE/s;

# Some specific fixes again
	$src =~ s/count &= 0xffff;/int count = uio->uio_resid & 0xfffff;/g;
	$src =~ s/file->f_flags/fflag/g;
	$src =~ s/UNIT\(file\)/ZAP_MINOR(dev)/g;
	$src =~ s/#define\s+ZAP_MINOR.*\n//;
	$src =~ s/ret \|= (POLL.*);/ret |= events & ($1);/g;
	$src =~ s/data \+= /arg += /g;
	$src =~ s/t = dev->si_drv1;/t = (struct zt_timer*)dev->si_drv1;/;
	$src =~ s/->file = file/->dev = dev/g;
	$src =~ s/->file = /->dev = /g;
	$src =~ s/void \*slab, \*ptr;/char *slab, *ptr;/g;
# Fix read/write functions to use uio
	$src =~ s/bcopy\(lindata, usrbuf \+ \(pos << 1\), pass << 1\);/if (uiomove((caddr_t)lindata, pass << 1, uio))\n\t\t\t\t\treturn EFAULT;/;
	$src =~ s/bcopy\(chan->readbuf\[res\], usrbuf, amnt\);/if (uiomove(chan->readbuf[res], amnt, uio))\n\t\t\t\t\treturn EFAULT;/;
	$src =~ s/bcopy\(usrbuf \+ \(pos << 1\), lindata, pass << 1\);/if (uiomove((caddr_t)lindata, pass << 1, uio))\n\t\t\t\t\treturn EFAULT;/;
	$src =~ s/bcopy\(usrbuf, chan->writebuf\[res\], amnt\);/if (uiomove(chan->writebuf[res], amnt, uio))\n\t\t\t\t\treturn EFAULT;/;
	$src =~ s/return amnt;/return 0;/g;
	# TASK_INIT for timer and channel
	$src =~ s/(\s+dev->si_drv1 = t;)/\n\tTASK_INIT(&t->selwakeup_task, 0, handle_selwakeup, &t->sel);$1/;
	$src =~ s/(\s+)(ZAP_LOCK_INIT\(chan->lock, "chan->lock"\);)/$1$2\n$1TASK_INIT(&chan->selwakeup_task, 0, handle_selwakeup, &chan->sel);/;

	$src =~ s/(\s+)(free\(t, M_ZAP\);)/$1taskqueue_drain(taskqueue_fast, &t->selwakeup_task);$1$2/;

#cleanout EXPORT_SYMBOL
	$src =~ s/(EXPORT_SYMBOL\([a-z_][a-z_0-9]+\);\n)+//gs;
	$src =~ s@#ifdef STANDALONE_ZAPATA.*/\* Get helper arithmetic \*/@#include "zaptel.h"\n\n/* Get helper arithmetic */@s;
	$src =~ s/static struct file_operations.*;\n//;
	$src =~ s/static struct file_operations.*[^}]+};\n//s;
	$src =~ s/static int schluffen.*?\n}/$zaptel_bits::schluffen/s;
	$src =~ s/static unsigned int zt_timer_poll.*?\n}/$zaptel_bits::zt_timer_poll/s;
	$src =~ s/static unsigned int\nzt_chan_poll.*?\n}/$zaptel_bits::zt_chan_poll/s;
	$src =~ s/(static int zt_init.*)/$zaptel_bits::zt_bsd_init\n$1/;


	$src =~ s/(new_maxspans = 0;)/ZAP_LOCK_DESTROY(span->lock);\n\t$1/;
	$src =~ s/static ssize_t zt_read/int zt_read/;
	$src =~ s/static ssize_t zt_write/int zt_write/;
	$src =~ s/static unsigned int zt_poll/static int zt_poll/;

	$src =~ s/(chan->channo = -1;\n)/$1$zaptel_bits::destroy_bsd_chandata/s;
	$src =~ s/static int zt_chan_ioctl.*?#if defined\(CONFIG_ZAPTEL_MMX\)/$zaptel_bits::prologue\n#if defined(CONFIG_ZAPTEL_MMX)/s;
	$src =~ s/(echo_can_shutdown\(\);)/$1\n\n$zaptel_bits::cleanup/;
	$src =~ s/([ \t]+)rwlock_init\(.*\);/$1res = zt_bsd_init();/;
	$src =~ s/t = samples\[x\] = ptr/t = samples\[x\] = (struct zt_tone *)ptr/;

	# Comment out per-channel check for O_NONBLOCK
	$src =~ s/(\s+)(.*chan->file.*)/$1\/* FIXME: $2 *\/$1printf("FIXME: per-channel check for O_NONBLOCK\\n");/g;
	$src =~ s/MODULE_PARM\((.*), "i"\)/TUNABLE_INT("zaptel.$1", \&$1)/g;

	# ioctl_load_zone is special. It should pass address because it points 
	# to variable-size region
	$src =~ s/bcopy\(arg, &th, sizeof\(th\)\);\n/bcopy\(arg, &arg, sizeof\(void *\)\);\n\tif(copyin(arg, &th, sizeof(th)))\n\t\treturn EFAULT;/;

# fixup prototypes
	foreach my $signature (keys %prototypes_fixup) {
		my $prototype = $prototypes_fixup{$signature};
		$src =~ s/$signature[^;{]+\n/$signature$prototype\n/g;
	}

# fixup arguments
	foreach my $signature (keys %args_fixup) {
		my $args = $args_fixup{$signature};
		$src =~ s/$signature/$args/g;
	}

	$src .= $zaptel_bits::epilogue;

	return $src;
} 
# }}}
sub zaptel_h_filter  # {{{
{
	my $zaptel_h = shift;

	$zaptel_h =~ s/_LINUX_ZAPTEL_H/_ZAPTEL_H/g;
	$zaptel_h =~ s/__KERNEL__/_KERNEL/g;
	$zaptel_h =~ s/unsigned long data/caddr_t arg/g;
	$zaptel_h =~ s/wait_queue_head_t(\s+)/void$1*/g;
	$zaptel_h =~ s/spinlock_t/lock_t/g;
	$zaptel_h = Beastify::cleanout_ifdefs($zaptel_h, "CONFIG_DEVFS_FS", 1); 
	$zaptel_h =~ s/void \*sel;.*/struct selinfo\tsel;\n\tstruct task\tselwakeup_task;/;
	$zaptel_h =~ s@(#ifdef\s+_KERNEL).*(#include "ecdis.h")@$1\n$zaptel_bits::zaptel_h_includes$2@s;
	$zaptel_h =~ s@#include\s+<linux/[a-z0-9_]+\.h>\s*\n@@g;
	#types 
	$zaptel_h = Beastify::port_types($zaptel_h);
	$zaptel_h =~ s/struct\s+file\s*\*\s*file/struct cdev *dev/;

	return $zaptel_h;
} 
# }}}
sub adt_lec_h_filter  # {{{
{
	my $src = shift;
	$src = Beastify::port_types($src);
	return $src;
} 
# }}}
sub adt_lec_c_filter  # {{{
{
	my $src = shift;
	$src = Beastify::port_types($src);
	$src = Beastify::port_errno($src);
	$src =~ s@linux/ctype@sys/ctype@g;
	return $src;
} 
# }}}
sub zconfig_h_filter # {{{
{
	my $zconfig_h = shift;
	$zconfig_h = Beastify::cleanout_ifdefs($zconfig_h, "__KERNEL__", 1); 
	$zconfig_h = Beastify::cleanout_if_with_str($zconfig_h, "LINUX_VERSION_CODE", 1); 

	return $zconfig_h;
}
# }}}
sub generic_ec_filter # {{{
{
	my $src = shift;

	$src =~ s/#include\s<linux\/.*\n//g;
	$src = Beastify::port_printk($src);

	# This is mostly for mg2ec.h
	$src =~ s/MALLOC/_MALLOC/g;
	$src =~ s/FREE/_FREE/g;
	$src =~ s/#define(\s+)_MALLOC.*\n.*/$zaptel_bits::mg2ec_h_malloc/;
	$src =~ s/void\s*\*/char */g;
	$src =~ s/ptr = ec;/ptr = (char *)ec;/;
	$src =~ s/_i = ptr;/_i = (int *)ptr;/g;
	$src =~ s/_s = ptr;/_s = (short *)ptr;/g;
	$src =~ s/memmove\((.*), ?(.*), ?(.*)\)/ovbcopy($2, $1, $3)/g;
	$src =~ s/short inline dc_removal/static inline short dc_removal/;
	$src =~ s/kfree/_FREE/g;

	return $src;
}
# }}}
sub ecdis_h_filter # {{{
{
	my $src = shift;
	$src =~ s/#define\s+(TRUE|FALSE).*\n//g;
	return $src;
}
# }}}
sub voicebus_h_filter # {{{
{
	my $src = shift;
	$src =~ s/u(8|16|32)/uint$1_t/g;
	$src =~ s/struct pci_dev ?\* ?/device_t /g;
	return $src;
}
# }}}
sub voicebus_c_filter # {{{
{
	my $src = shift;
	my $voicebus_c_includes = join  "\n", (
		"#include <sys/param.h>  /* defines used in kernel.h */",
		"#include <sys/module.h>", 
		"#include <sys/systm.h>", 
		"#include <sys/bus.h>", 
		"#include <sys/endian.h>", 
		"#include <sys/kernel.h>", 
		"#include <sys/lock.h>", 
		"#include <sys/mutex.h>", 
		"#include <sys/queue.h>", 
		"#include <vm/uma.h>", 
		"#include <sys/condvar.h>", 
		"#include <machine/bus.h>", 
		"#include <machine/resource.h>", 
		"#include <dev/pci/pcireg.h>",
		"#include <dev/pci/pcivar.h>",
		'#include "zaptel-compat.h"',
		'#include "voicebus-compat.h"',
		'',
		'MALLOC_DEFINE(M_VB, "voicebus", "Voicbus data structures");'
	);

	$src =~ s/u(8|16|32)/uint$1_t/g;
	$src =~ s/struct pci_dev ?\* ?/device_t /g;
	$src =~ s/#include <linux.*timer.h>/$voicebus_c_includes/s;
	$src =~ s/dma_addr_t/bus_addr_t/g;
	$src =~ s/#define LOCKS_VOICEBUS.*/#define LOCKS_VOICEBUS/g;
	$src =~ s/KERN_##_lvl //g;
	$src =~ s/wmb\(\);/\/* FIXME: wmb *\//g;
	# fix locks
	$src = Beastify::port_locks($src);
	$src = Beastify::port_errno($src);
	$src = Beastify::port_printk($src);
	$src = Beastify::port_kmemory($src, "M_VB");
	$src = Beastify::cleanout_if_with_str($src, "LINUX_VERSION_CODE"); 
	$src =~ s/struct voicebus_descriptor\* desc;\n//;
	$src =~ s/bus_addr_t\s+desc_dma/struct vb_dma_t\tvb_dma/g;
	$src =~ s/\S+ = pci_alloc_consistent\(([^,]*?),\s*([^,]*?),\s*([^,]*?)\);/vb_dma_allocate($1, &dl->vb_dma, $2);/gs;
	$src =~ s/pci_free_consistent\(([^,]*?),\s*([^,]*?),\s*([^,]*?),([^,]*?)\);/vb_dma_destroy(&dl->vb_dma);/gs;
	# XXX pdev hardcoded
	$src =~ s/(\S+) = dma_map_single\(([^,]*?),\s*([^,]*?),\s*([^,]*?),\s*([^,]*?)\);/vb_dma_map(vb->pdev, &dl->vb_dma, $3, $4);/g;
	$src =~ s/dma_unmap_single\(([^,]*?),\s*([^,]*?),\s*([^,]*?),\s*([^,]*?)\);/vb_dma_unmap(vb->pdev, &dl->vb_dma);/g;
	$src = Beastify::port_pci_stuff($src);
	$src = Beastify::port_jiffies($src);
	$src =~ s/(->|\.)desc/$1vb_dma.desc/g;
	$src =~ s/GFP_KERNEL/M_ZERO | M_NOWAIT/g;
	$src =~ s/GFP_ATOMIC/M_ZERO | M_NOWAIT/g;

	$src =~ s/kmem_cache_create\(([^,]*?),\s*([^,]*?),\s*([^,]*?),\s*([^,]*?),\s*([^,]*?)\);/uma_zcreate((char *)$1, $2, NULL, NULL, NULL, NULL, 0, 0);/g;
	$src =~ s/kmem_cache_alloc/uma_zalloc/g;
	$src =~ s/kmem_cache_destroy/uma_zdestroy/g;
	$src =~ s/struct kmem_cache\s*\*\s*/uma_zone_t /g;
	$src =~ s/kmem_cache_free/uma_zfree/g;
	$src =~ s/unsigned long(\s+vb_wait_for_completion_timeout)/static int$1/g;
	$src =~ s/void(\s+__vb_increase_latency)/static void$1/g;
	$src =~ s/INIT_COMPLETION\(/init_completion(&/g;
	$src =~ s/release_region.*/vb_unmap_iospace(vb->pdev, &vb->vb_io);/g;
	$src =~ s/free_irq.*/vb_teardown_irq(vb->pdev, &vb->vb_irq);/g;
	$src =~ s/uint32_t iobase.*?int irq;/struct vb_io_t vb_io;\n\tstruct vb_irq_t vb_irq;/s;
	$src =~ s/outb\((.*), \w+->io(addr|base) \+ (.*)\);/VB_WRITE1($3, $1);/g;
	$src =~ s/outl\((.*),\s+\w+->io(addr|base) \+ (.*)\);/VB_WRITE4($3, $1);/g;
	$src =~ s/inb\(\w+->io(addr|base) \+ /VB_READ1(/g;
	$src =~ s/inl\(\w+->io(addr|base) \+ /VB_READ4(/g;
	$src =~ s/(pci_disable_device.*)/\/* XXX: $1 *\//;
	# Fixup ISR routine
	$src =~ s/static irqreturn_t[^{]*{/ZAP_IRQ_HANDLER(vb_isr)\n{/g;
	$src = Beastify::port_interrupt_code($src);

	# PCI stuff
	$src =~ s/if\s*\(pci_enable_device\(pdev\)\)(\s*{?)/if (pci_enable_io(pdev, SYS_RES_IOPORT))$1/;
	$src =~ s/[^\n]*vb->io(addr|base) = pci_resource_start.*\n//;
	$src =~ s/NULL == request_region\([^\)]+\)/vb_map_iospace(pdev, \&vb->vb_io)/;
	$src =~ s/(\s+).*pci_alloc_consistent\(pdev, (.*), .*\);/$1vb_dma_allocate(pdev, &vb->vb_dma, $2);/;
	$src =~ s/release_region.*/vb_unmap_iospace(vb->dev, &vb->vb_io);/g;
	$src =~ s/free_irq.*/vb_teardown_irq(vb->dev, &vb->vb_irq);/g;
	$src =~ s/pci_free_consistent.*/vb_dma_destroy(&vb->vb_dma);/g;
	$src =~ s/pci_set_master/pci_enable_busmaster/g;
	# $src =~ s/.*\n.*pci_set_drvdata.*\n//g;
	# $src =~ s/pci_get_drvdata/device_get_softc/g;
	$src =~ s/request_irq\([^,]+, ([^,]+),[^\)]+\)/vb_setup_irq(pdev, &vb->vb_irq, $1, vb)/;


	return $src;
}
# }}}
# {{{ Files to port for zaptel package
%files_to_copy = (
	"kernel/fasthdlc.h" => "zaptel/fasthdlc.h",
       	"kernel/biquad.h"=> "zaptel/biquad.h",
	"kernel/arith.h"=> "zaptel/arith.h",
	"kernel/digits.h"=> "zaptel/digits.h",
	"kernel/mg2ec_const.h"=> "zaptel/mg2ec_const.h",
	"kernel/proslic.h"=> "common/proslic.h",
	"kernel/fxo_modes.h"=> "common/fxo_modes.h",
);

%files_to_beastify = (
	"kernel/zaptel-base.c" => { 
		filter => \&zaptel_base_filter,
		dst => "zaptel/zaptel-base.c"
	},
	"kernel/zaptel.h" => {
		filter => \&zaptel_h_filter,
		dst => "zaptel/zaptel.h"
	},
	"kernel/zconfig.h" => {
		filter => \&zconfig_h_filter,
		dst => "zaptel/zconfig.h"
	},
	"kernel/ecdis.h" => {
		filter => \&ecdis_h_filter,
		dst => "zaptel/ecdis.h"
	},
	"kernel/mg2ec.h" => {
		filter => \&generic_ec_filter,
		dst => "zaptel/mg2ec.h"
	},
	"kernel/jpah.h" => {
		filter => \&generic_ec_filter,
		dst => "zaptel/jpah.h"
	},
	"kernel/kb1ec.h" => {
		filter => \&generic_ec_filter,
		dst => "zaptel/kb1ec.h"
	},
	"kernel/sec.h" => {
		filter => \&generic_ec_filter,
		dst => "zaptel/sec.h"
	},
	"kernel/sec-2.h" => {
		filter => \&generic_ec_filter,
		dst => "zaptel/sec-2.h"
	},
	"kernel/adt_lec.h" => {
		filter => \&adt_lec_h_filter,
		dst => "common/adt_lec.h"
	},
	"kernel/adt_lec.c" => {
		filter => \&adt_lec_c_filter,
		dst => "common/adt_lec.c"
	},
	"kernel/voicebus.h" => {
		filter => \&voicebus_h_filter,
		dst => "common/voicebus.h"
	},
	"kernel/voicebus.c" => {
		filter => \&voicebus_c_filter,
		dst => "common/voicebus.c"
	},

);
# }}}

1;
