# Some chunks of code required by beastify.pl
package zaptel_bits;

our ($includes, $prologue, $epilogue, $cleanup, $zt_bsd_init);
our ($destroy_bsd_chandata, $schluffen, $zt_timer_poll, $zt_chan_poll);
our ($zaptel_h_includes, $mg2ec_h_malloc);

$includes = <<__EOF__;
#include <sys/types.h>
#include <sys/ctype.h>
#include <sys/uio.h>
#include <sys/param.h>
#include <sys/select.h>
#include <sys/poll.h>
#include <sys/selinfo.h>
#include <sys/systm.h>
#include <sys/malloc.h>
#include <sys/kernel.h>
#ifdef __FreeBSD__
#include <sys/bus.h>
#include <sys/module.h>
#endif
#include <sys/lock.h>
#include <sys/condvar.h>
#if __FreeBSD_version >= 700000
#include <sys/rwlock.h>
#endif
#include <sys/file.h>
#include <sys/filedesc.h>
#include <sys/fcntl.h>
#include <sys/ioccom.h>
#include <sys/proc.h>
#include <net/ppp_defs.h>
#include <sys/conf.h>
#include <sys/stat.h>

#include <sys/sysctl.h>
#include <sys/filio.h>
#include <sys/kthread.h>
#include <sys/taskqueue.h>

#include <machine/bus.h>

__EOF__

$prologue = <<__EOF__;
#define ZAPTEL_VERSION "zaptel-bsd-ng v0.0.1"
MODULE_VERSION(zaptel, 1);
MALLOC_DEFINE(M_ZAP, "zaptel", "ZapTel interface data structures");

static d_open_t		zt_open;
static d_close_t	zt_close;
static d_read_t		zt_read;
static d_write_t	zt_write;
static d_ioctl_t	zt_ioctl;
static d_poll_t		zt_poll;
static d_mmap_t		zt_mmap;

/* forward declaration */
static int zt_chan_ioctl(struct cdev *, u_long, caddr_t, int, int);

static struct cdev * dev_ctl;
static struct cdevsw zt_devsw = {
	.d_version	= D_VERSION,
	.d_open		= zt_open,
	.d_close	= zt_close,
	.d_read		= zt_read,
	.d_write	= zt_write,
	.d_ioctl	= zt_ioctl,
	.d_poll		= zt_poll,
	.d_mmap		= zt_mmap,
	.d_name		= "zaptel"
};

static eventhandler_tag zt_ehtag;
static struct clonedevs	*all_clones;
static struct clonedevs	*channel_clones;
static struct clonedevs	*pseudo_clones;
static struct clonedevs	*timer_clones;
static struct clonedevs	*transcode_clones;
static struct clonedevs	*channels_clones;
/* If si_drv2 is absent - it's ctl */
#define ZAP_MINOR(dev) (dev)->si_drv2 ? \\
    ((struct clone_dev_info *)(dev)->si_drv2)->minor : ZAP_MINOR_CTL

/* 
 * Structure to keep both unit number and file flags, 
 */
struct clone_dev_info
{
	/* 
	 * Type of device. Linux counts on a minor number we 
	 * should keep in swdev structure for there is no way to
	 * have two devices with the same <major, minor> pair.
	 */
	int minor;		
	unsigned int f_flags;
};

static void 
handle_selwakeup(void *context, int pending) 
{
	struct selinfo * sel = context;
	selwakeup(sel);
}
__EOF__

$zt_bsd_init = <<__EOF__;
static int 
zt_bsd_init(void)
{
	int unit = -1;

	/* 
	 * A kind of hack. Without any pre-created device 
	 * zt_clone will get "zap" string as name instead of 
	 * "zap/timer" or something
	 */
	dev_ctl = NULL;

	if (clone_create(&all_clones, &zt_devsw, &unit, &dev_ctl, 0)) {
		dev_ctl = make_dev(&zt_devsw, 
		    unit2minor(unit),
		    UID_ROOT, GID_WHEEL, 0644, "zap/ctl");
	}

	if(dev_ctl) {
		dev_ref(dev_ctl);
		dev_ctl->si_flags |= SI_CHEAPCLONE;
	} else {
		printf("Failed to create zap/ctl");
		return ENXIO;
	}

	ZAP_RWLOCK_INIT(chan_lock, "chanlock");
	ZAP_RWLOCK_INIT(zone_lock, "zone_lock");
	ZAP_LOCK_INIT(zaptimerlock, "zaptimerlock");
	ZAP_LOCK_INIT(bigzaplock, "bigzaplock");

	return 0;
}
__EOF__

$epilogue = <<__EOF__;
/* Clone device */
static void
zt_clone(void *arg, struct ucred *cred, char *name, int namelen,
    struct cdev **dev)
{
	int i, unit;
	unsigned int dev_minor;
	struct clonedevs **clones;
	struct clone_dev_info * info;

	char dev_name[20];
	char dev_format[20];

	if (*dev != NULL)
		return;

	/* check if it is our device */
	if (!strcmp(name, "zap/channel")) {
		dev_minor = ZAP_MINOR_CHANNEL;
		clones = &channel_clones;
		strcpy(dev_format, "zap/channel%d");
		strncpy(dev_name, name, sizeof(dev_name) - 1);
	}
	else if (!strcmp(name, "zap/pseudo")) {
		dev_minor = ZAP_MINOR_PSEUDO;
		clones = &pseudo_clones;
		strcpy(dev_format, "zap/pseudo%d");
		strncpy(dev_name, name, sizeof(dev_name) - 1);
	}
	else if (!strcmp(name, "zap/timer")) {
		dev_minor = ZAP_MINOR_TIMER;
		clones = &timer_clones;
		strcpy(dev_format, "zap/timer%d");
		strncpy(dev_name, name, sizeof(dev_name) - 1);
	}
	else if (!strcmp(name, "zap/transcode")) {
		return;
		/*
		   Sorry, no transcode stuff yet
		dev_minor = ZAP_MINOR_TRANSCODE;
		clones = &transcode_clones;
		strcpy(dev_format, "zap/transcode%d");
		strncpy(dev_name, name, sizeof(dev_name) - 1);
		*/
	}
	else if (!strncmp(name, "zap/", 4))
	{
		for (i = 4; i < strlen(name); i++)
			if (!isdigit(name[i]))
				return;
		dev_minor = strtol(name + 4, NULL, 10);
		if ((dev_minor >= 250) || !dev_minor)
		{
			printf("Wrong minor for character device: %d\\n",
			    dev_minor);
			return;
		}
		clones = &channels_clones;
		sprintf(dev_format, "zap/chan%d.%%d", dev_minor);
		sprintf(dev_name, "zap/chan%d", dev_minor);
	}
	else 
		return;

	clones = &all_clones;
	printf("Trying %s(%d) dev == %p\\n", name, namelen, *dev);

	unit = -1;
	printf("creating clone (%p)\\n", clones);
	i = clone_create(clones, &zt_devsw, &unit, dev, 0);
	printf("Clone created\\n");

	if (i) {
		printf("Makeing device ");
		printf("%s(%d)", dev_format, unit);
		printf(" with minor %d\\n", dev_minor);
		*dev = make_dev(&zt_devsw, 
		    unit2minor(unit),
		    UID_ROOT, GID_WHEEL, 0644, dev_format, unit);
		printf("Device created\\n");
	}

	if(*dev)
	{
		dev_ref(*dev);
		info = malloc(sizeof(struct clone_dev_info), M_ZAP, M_NOWAIT);
		if(!info)
			return;
		(*dev)->si_flags |= SI_CHEAPCLONE;
		info->minor = dev_minor;
		info->f_flags = 0;
		(*dev)->si_drv2 = (void *)info;
		(*dev)->si_drv1 = NULL;
	}

        return;
}

static int
ztdev_load(module_t mod, int cmd, void *arg)
{
	int  err = 0;

	switch (cmd) {
	case MOD_LOAD:
		clone_setup(&all_clones);
		zt_init();
		clone_setup(&channel_clones);
		clone_setup(&pseudo_clones);
		clone_setup(&timer_clones);
		clone_setup(&transcode_clones);
		clone_setup(&channels_clones);
		zt_ehtag = EVENTHANDLER_REGISTER(dev_clone, zt_clone, 0, 1000);
		break;		  /* Success*/

	case MOD_UNLOAD:
		zt_cleanup();
		if (zt_ehtag != NULL)
			EVENTHANDLER_DEREGISTER(dev_clone, zt_ehtag);
		clone_cleanup(&channel_clones);
		clone_cleanup(&pseudo_clones);
		clone_cleanup(&timer_clones);
		clone_cleanup(&transcode_clones);
		clone_cleanup(&channels_clones);
		clone_cleanup(&all_clones);
		break;		  /* Success*/

	default:	/* we only understand load/unload*/
		err = EINVAL;
		break;
	}

	return(err);
}

/* Now declare the module to the system */
DEV_MODULE(ztdev, ztdev_load, NULL);

#ifdef CONFIG_ZAPATA_NET
MODULE_DEPEND (ztp, sppp, 1, 1, 1);
#endif
__EOF__

$destroy_bsd_chandata =<<__EOF__;
	ZAP_LOCK_DESTROY(chan->lock);
	taskqueue_drain(taskqueue_fast, &chan->selwakeup_task);
__EOF__

$schluffen = <<__EOF__;
int schluffen(void *q)
{
	int rc = tsleep(q, PZERO | PCATCH, "schluffen", 0);

	switch(rc)
	{
		case EINTR:
		case ERESTART:
			return rc;
	}

	return 0;
}
__EOF__

$cleanup = <<__EOF__;
	if (dev_ctl)
		destroy_dev(dev_ctl);

	ZAP_RWLOCK_DESTROY(chan_lock);
	ZAP_RWLOCK_DESTROY(zone_lock);
	ZAP_LOCK_DESTROY(zaptimerlock);
	ZAP_LOCK_DESTROY(bigzaplock);
__EOF__

$zaptel_h_includes = <<__EOF__;
#include <sys/selinfo.h>
#include <sys/taskqueue.h>
 
#include "zconfig.h"
#include "zaptel-compat.h"
__EOF__

$mg2ec_h_malloc = <<__EOF__;
#ifdef _KERNEL
#define _MALLOC(a) malloc((a), M_DEVBUF, M_NOWAIT)
#define _FREE(a) free(a, M_DEVBUF)
#else
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <string.h>
#include <math.h>
#define _MALLOC(a) malloc(a)
#define _FREE(a) free(a)
#endif
__EOF__

$zt_timer_poll = <<__EOF__;
static unsigned int zt_timer_poll(struct cdev *dev, int events, 
    struct thread *td)
{
	struct zt_timer *timer = dev->si_drv1;
	int ret = 0;
	int will_selrecord = 0;

	if (timer) {
		ZAP_LOCK(zaptimerlock);
		if (timer->tripped || timer->ping) 
			ret |= events & POLLPRI;
		else
			will_selrecord = 1;
		ZAP_UNLOCK(zaptimerlock);

		if (will_selrecord)
			selrecord(td, &timer->sel);

	} else
		ret = EINVAL;
	return ret;
}
__EOF__

$zt_chan_poll = <<__EOF__;
static unsigned int
zt_chan_poll(struct cdev *dev, int events, struct thread *td, int unit)
{   
	
	struct zt_chan *chan = chans[unit];
	int	ret;

	  /* do the poll wait */
	if (chan) {
		ret = 0; /* start with nothing to return */
		ZAP_LOCK(chan->lock);
		   /* if at least 1 write buffer avail */
		if (chan->inwritebuf > -1) {
			ret |= events & (POLLOUT | POLLWRNORM);
		}
		if ((chan->outreadbuf > -1) && !chan->rxdisable) {
			ret |= events & (POLLIN | POLLRDNORM);
		}
		if (chan->eventoutidx != chan->eventinidx)
		   {
			/* Indicate an exception */
			ret |= events & POLLPRI;
		   }
		ZAP_UNLOCK(chan->lock);

		if(ret == 0)
			/* do the poll wait */
			selrecord(td, &chan->sel);

	} else
		ret = EINVAL;
	return(ret);  /* return what we found */
}
__EOF__

1;
