/*-
 * Copyright (C) 2007 by Oleksandr Tymoshenko. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR OR HIS RELATIVES BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF MIND, USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: $
 *
 */
#ifndef __VOICEBUS_COMPAT_H__
#define __VOICEBUS_COMPAT_H__

/* 
 * Generic DMA struct for all HW cards. descrip fields are used only
 */ 
struct vb_dma_t {
	struct voicebus_descriptor* desc;
	bus_addr_t	desc_dma;

	bus_dma_tag_t	dma_tag;
	bus_dma_tag_t	dma_buffers_tag;
	bus_dmamap_t	dma_map;
};

struct vb_io_t {
        /* newbus stuff */
        struct resource *       base_addr_res;
	int			base_addr_rid;
        bus_space_tag_t         base_addr_tag;
        bus_space_handle_t      base_addr_handle;
};

struct vb_irq_t {
        struct resource		*irq_res;
        int			irq_rid;
        void			*irq_cookie;
        bus_space_tag_t		irq_tag;
        bus_space_handle_t	irq_handle;
};

/* foo definitions */
#define DMA_TO_DEVICE 0
#define DMA_FROM_DEVICE 0

/* Naming convention should be preserved */
#define VB_READ1(off)      bus_space_read_1(vb->vb_io.base_addr_tag, \
				vb->vb_io.base_addr_handle, (off))
#define VB_WRITE1(off,v)   bus_space_write_1(vb->vb_io.base_addr_tag, \
				vb->vb_io.base_addr_handle, (off), (v))
#define VB_READ4(off)      bus_space_read_4(vb->vb_io.base_addr_tag, \
				vb->vb_io.base_addr_handle, (off))
#define VB_WRITE4(off,v)   bus_space_write_4(vb->vb_io.base_addr_tag, \
				vb->vb_io.base_addr_handle, (off), (v))

/* IRQ handler */
#if __FreeBSD_version < 700031
#define VB_IRQ_HANDLER(a) static void a(void *dev_id)
#define vb_irq_handler_t driver_intr_t
#else
#define VB_IRQ_HANDLER(a) static int a(void *dev_id)
#define vb_irq_handler_t driver_filter_t
#endif

int vb_setup_irq(device_t, struct vb_irq_t*, vb_irq_handler_t, void *);
void vb_teardown_irq(device_t, struct vb_irq_t *);
int vb_map_iospace(device_t, struct vb_io_t *);
void vb_unmap_iospace(device_t, struct vb_io_t *);
int vb_dma_allocate(device_t, struct vb_dma_t *, bus_size_t, bus_size_t);
bus_addr_t vb_dma_map(device_t, struct vb_dma_t *, bus_dmamap_t *,
    void **, bus_size_t);
void vb_dma_unmap(device_t, struct vb_dma_t *, void *, bus_dmamap_t);
void vb_dma_destroy(struct vb_dma_t *);

#endif // __VOICEBUS_COMPAT_H__
